@@ -22,6 +22,7 @@ import daft
 from graphreduce.node import GraphReduceNode, DynamicNode, SQLNode
 from graphreduce.enum import ComputeLayerEnum, PeriodUnit
 from graphreduce.storage import StorageClient
+from graphreduce.models import sqlop
 
 logger = get_logger("GraphReduce")
 
@@ -56,7 +57,7 @@ class GraphReduce(nx.DiGraph):
             "categorical": ["count", "nunique"],
             "embedding": ["first"],
             "image_embedded": ["first"],
-            "multicategorical": ["mode"],
+            "multicategorical": ["first"],
             "sequence_numerical": ["min", "max"],
             "timestamp": ["min", "max"],
         },
@@ -79,6 +80,7 @@ class GraphReduce(nx.DiGraph):
         # Debug
         debug: bool = False,
         checkpoint_schema: str = None,
+        date_filters_on_agg: bool = False,
         *args,
         **kwargs,
     ):
@@ -106,6 +108,7 @@ class GraphReduce(nx.DiGraph):
             storage_client: optional `graphreduce.storage.StorageClient` instance to checkpoint compute graphs
             catalog_client: optional Unity or Polaris catalog client instance
             debug: bool whether to run debug logging
+            date_filters_on_agg: bool whether or not to automatically filter by dates during custom defined aggregations
         """
         super(GraphReduce, self).__init__(*args, **kwargs)
 
@@ -132,6 +135,8 @@ class GraphReduce(nx.DiGraph):
         self.auto_feature_hops_front = auto_feature_hops_front
         self.feature_typefunc_map = feature_typefunc_map
         self.feature_stype_map = feature_stype_map
+        self.date_filters_on_agg = date_filters_on_agg
+
 
         # SQL dialect parameters.
         self._lazy_execution = lazy_execution
@@ -533,6 +538,7 @@ class GraphReduce(nx.DiGraph):
             if relation_node._cur_data_ref
             else relation_node.fpath
         )
+        logger.info(f"parent table: {parent_table} relation table: {relation_table}")
         # Check if the relation foreign key is already
         # in the parent and, if so, rename it.
         parent_samp = parent_node.get_sample()
@@ -571,6 +577,8 @@ class GraphReduce(nx.DiGraph):
             schema=self._checkpoint_schema,
             dry=self.dry_run,
         )
+        # Get the table after the join.
+        parent_samp = parent_node.get_sample()
         self.sql_ops.append(JOIN_SQL)
         if parent_node._ref_sql:
             self.sql_ops.append(parent_node._ref_sql)
@@ -833,9 +841,13 @@ class GraphReduce(nx.DiGraph):
                         if relation_node.prep_for_features()
                         else []
                     )
-                    reduce_ops = tfilt + relation_node.do_reduce(
-                        edge_data["relation_key"]
-                    )
+                    # NOTE: we do not automatically do date filtering
+                    # here so maybe that should be a top-level parameter
+                    # for when we have a custom reduce implementation?
+                    reduce_ops = relation_node.do_reduce(edge_data["relation_key"])
+                    if self.date_filters_on_agg:
+                        reduce_ops = reduce_ops + tfilt
+                        logger.info(f"Added in date filtering ops: {tfilt}")
                     reduce_sql = relation_node.build_query(reduce_ops)
                     logger.info(f"reduce SQL: {reduce_sql}")
                     self.sql_ops.append(reduce_sql)
@@ -861,9 +873,13 @@ class GraphReduce(nx.DiGraph):
             )
 
             # Target variables.
-            if self.label_node and (
-                self.label_node == relation_node
-                or relation_node.label_field is not None
+            if (
+                self.label_node
+                and (
+                    self.label_node == relation_node
+                    or relation_node.label_field is not None
+                )
+                or relation_node.do_labels(edge_data["relation_key"]) is not None
             ):
                 logger.info(f"Had label node {self.label_node}")
 
@@ -951,12 +967,19 @@ class GraphReduce(nx.DiGraph):
 
             # post-join annotations (if any)
             pja_sql = parent_node.build_query(parent_node.do_post_join_annotate())
+            logger.info(f"Running do_post_join_annotate")
+            logger.info(f"{pja_sql}")
             self.sql_ops.append(pja_sql)
             pja_ref = parent_node.create_ref(
                 pja_sql,
                 parent_node.do_post_join_annotate,
                 schema=self._checkpoint_schema,
                 dry=self.dry_run,
+                # Need to ensure we're overwriting the
+                # reference because previous executions
+                # may have tried creating a reference before
+                # dependencies were merged.
+                overwrite=True,
             )
             # post-join filters (if any)
             if hasattr(parent_node, "do_post_join_filters"):
@@ -967,8 +990,17 @@ class GraphReduce(nx.DiGraph):
                     parent_node.do_post_join_filters,
                     schema=self._checkpoint_schema,
                     dry=self.dry_run,
+                    # Need to ensure we're overwriting the
+                    # reference because previous executions
+                    # may have tried creating a reference before
+                    # dependencies were merged.
+                    overwrite=True,
                 )
 
+            # post-join reduce (if any)
+            if hasattr(parent_node, "do_post_join_reduce"):
+                pass
+
     def do_transformations(self):
         """
         Perform all graph transformations
