@@ -105,7 +105,7 @@ def test_custom_node_graph():
     ic(gr.parent_node.df.head())
     assert len(gr.parent_node.df) == 4
 
-   
+
 
 
 def test_dynamic_node_instance():
@@ -330,22 +330,22 @@ class CustNode(SQLNode):
         return [
             sqlop(optype=SQLOpType.select, opval=f"*, LENGTH({self.colabbr('name')}) as {self.colabbr('name_length')}")
         ]
-    
+
     def do_filters(self) -> typing.Union[sqlop, typing.List[sqlop]]:
         return [
             sqlop(optype=SQLOpType.where, opval=f"{self.colabbr('id')} < 3")
         ]
-    
+
     def do_normalize(self):
         pass
-    
-    
+
+
     def do_reduce(self, reduce_key):
         pass
-    
+
     def do_post_join_annotate(self):
         pass
-    
+
     def do_post_join_filters(self):
         pass
 
@@ -596,7 +596,7 @@ def test_duckdb_graph_noreduce():
             pk='id',
             columns=['id','name'],
             table_name='customer'
-            )    
+            )
     orders = DuckdbNode(
             fpath=f"'{os.path.join(data_path, 'orders.csv')}'",
             prefix='ord',
@@ -637,7 +637,7 @@ def test_duckdb_graph_reduce():
             pk='id',
             columns=['id','name'],
             table_name='customer'
-            )    
+            )
     orders = DuckdbNode(
             fpath=f"'{os.path.join(data_path, 'orders.csv')}'",
             prefix='ord',
@@ -672,3 +672,76 @@ def test_duckdb_graph_reduce():
     ic(res.shape)
     assert res.shape[0] == 4
     con.close()
+
+
+def test_duckdb_join_deps():
+    con = duckdb.connect()
+    orders = DuckdbNode(
+            fpath=f"'{os.path.join(data_path, 'orders.csv')}'",
+            prefix='ord',
+            pk='id',
+            date_key='ts',
+            columns=['id','customer_id','ts', 'amount'],
+            table_name='orders',
+            do_reduce_ops=[
+                sqlop(optype=SQLOpType.agg, opval="ord_customer_id"),
+                sqlop(optype=SQLOpType.aggfunc, opval="count(ord_id) as ord_num_orders")
+                ]
+            )
+    notif = DuckdbNode(
+            fpath=f"'{os.path.join(data_path, 'notifications.csv')}'",
+            prefix='notif',
+            pk='id',
+            date_key='ts',
+            columns=['id','customer_id','ts'],
+            table_name='notifications',
+            do_reduce_ops=[
+                sqlop(optype=SQLOpType.agg, opval="notif_customer_id"),
+                sqlop(optype=SQLOpType.aggfunc, opval="count(notif_id) as notif_num_notifications")
+                ]
+            )
+    cust = DuckdbNode(
+            fpath=f"'{os.path.join(data_path, 'cust.csv')}'",
+            prefix='cust',
+            pk='id',
+            columns=['id','name'],
+            table_name='customer',
+            do_post_join_annotate_ops=[
+                sqlop(optype=SQLOpType.select, opval="*"),
+                sqlop(optype=SQLOpType.select, opval="notif_num_notifications / ord_num_orders as notifs_per_order")
+                ],
+            do_post_join_filters_ops=[
+                sqlop(optype=SQLOpType.where, opval="notifs_per_order >= 2")
+                ],
+            do_post_join_annotate_requires=[orders,notif],
+            do_post_join_filters_requires=[orders, notif]
+            )
+
+    gr = GraphReduce(
+        name='duckdb test',
+        parent_node=cust,
+        compute_period_val=365,
+        compute_period_unit=PeriodUnit.day,
+        cut_date=datetime.datetime(2023, 5, 1),
+        auto_features=True,
+        auto_labels=True,
+        label_node=orders,
+        label_field='id',
+        label_operation='count',
+        label_period_val=90,
+        label_period_unit=PeriodUnit.day,
+        compute_layer=ComputeLayerEnum.duckdb,
+        sql_client=con
+        )
+    gr.add_node(cust)
+    gr.add_node(orders)
+    gr.add_node(notif)
+    gr.add_entity_edge(parent_node=cust,relation_node=orders,parent_key='id',relation_key='customer_id',reduce=True)
+    gr.add_entity_edge(parent_node=cust,relation_node=notif,parent_key='id',relation_key='customer_id',reduce=True)
+    gr.do_transformations_sql()
+    res = con.sql(f"select * from {gr.parent_node._cur_data_ref}").to_df()
+    ic(res)
+    ic(res.columns)
+    ic(res.shape)
+    assert res.shape[0] == 2
+
